@@ -15,35 +15,30 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <cstdlib>
-#include <stdio.h>
-
 #include "Add.h"
 
-#include "cli/Generate.h"
-#include "cli/TextStream.h"
-#include "cli/Utils.h"
-#include "core/Database.h"
-#include "core/Entry.h"
+#include "Generate.h"
+#include "Utils.h"
 #include "core/Group.h"
 #include "core/PasswordGenerator.h"
 
-const QCommandLineOption Add::UsernameOption = QCommandLineOption(QStringList() << "u"
-                                                                                << "username",
+#include <QCommandLineParser>
+
+const QCommandLineOption Add::UsernameOption = QCommandLineOption(QStringList() << "u" << "username",
                                                                   QObject::tr("Username for the entry."),
                                                                   QObject::tr("username"));
 
 const QCommandLineOption Add::UrlOption =
     QCommandLineOption(QStringList() << "url", QObject::tr("URL for the entry."), QObject::tr("URL"));
 
-const QCommandLineOption Add::PasswordPromptOption =
-    QCommandLineOption(QStringList() << "p"
-                                     << "password-prompt",
-                       QObject::tr("Prompt for the entry's password."));
+const QCommandLineOption Add::NotesOption =
+    QCommandLineOption(QStringList() << "notes", QObject::tr("Notes for the entry."), QObject::tr("Notes"));
 
-const QCommandLineOption Add::GenerateOption = QCommandLineOption(QStringList() << "g"
-                                                                                << "generate",
-                                                                  QObject::tr("Generate a password for the entry."));
+const QCommandLineOption Add::PasswordPromptOption =
+    QCommandLineOption(QStringList() << "p" << "password-prompt", QObject::tr("Prompt for the entry's password."));
+
+const QCommandLineOption Add::GenerateOption =
+    QCommandLineOption(QStringList() << "g" << "generate", QObject::tr("Generate a password for the entry."));
 
 Add::Add()
 {
@@ -51,6 +46,7 @@ Add::Add()
     description = QObject::tr("Add a new entry to a database.");
     options.append(Add::UsernameOption);
     options.append(Add::UrlOption);
+    options.append(Add::NotesOption);
     options.append(Add::PasswordPromptOption);
     positionalArguments.append({QString("entry"), QObject::tr("Path of the entry to add."), QString("")});
 
@@ -65,6 +61,7 @@ Add::Add()
     options.append(Generate::ExcludeCharsOption);
     options.append(Generate::ExcludeSimilarCharsOption);
     options.append(Generate::IncludeEveryGroupOption);
+    options.append(Generate::CustomCharacterSetOption);
 }
 
 int Add::executeWithDatabase(QSharedPointer<Database> database, QSharedPointer<QCommandLineParser> parser)
@@ -77,7 +74,7 @@ int Add::executeWithDatabase(QSharedPointer<Database> database, QSharedPointer<Q
 
     // Cannot use those 2 options at the same time!
     if (parser->isSet(Add::GenerateOption) && parser->isSet(Add::PasswordPromptOption)) {
-        err << QObject::tr("Cannot generate a password and prompt at the same time!") << endl;
+        err << QObject::tr("Cannot generate a password and prompt at the same time.") << endl;
         return EXIT_FAILURE;
     }
 
@@ -105,6 +102,10 @@ int Add::executeWithDatabase(QSharedPointer<Database> database, QSharedPointer<Q
         entry->setUrl(parser->value(Add::UrlOption));
     }
 
+    if (!parser->value(Add::NotesOption).isEmpty()) {
+        entry->setNotes(parser->value(Add::NotesOption).replace("\\n", "\n"));
+    }
+
     if (parser->isSet(Add::PasswordPromptOption)) {
         if (!parser->isSet(Command::QuietOption)) {
             out << QObject::tr("Enter password for new entry: ") << flush;
@@ -117,7 +118,7 @@ int Add::executeWithDatabase(QSharedPointer<Database> database, QSharedPointer<Q
     }
 
     QString errorMessage;
-    if (!database->save(&errorMessage, true, false)) {
+    if (!database->save(Database::Atomic, {}, &errorMessage)) {
         err << QObject::tr("Writing the database failed %1.").arg(errorMessage) << endl;
         return EXIT_FAILURE;
     }
