@@ -28,8 +28,10 @@ public:
     int executeWithDatabase(QSharedPointer<Database> db, QSharedPointer<QCommandLineParser> parser);
 
     static const QCommandLineOption TotpOption;
+    static const QCommandLineOption AllAttributesOption;
     static const QCommandLineOption AttributesOption;
     static const QCommandLineOption ProtectedAttributesOption;
+    static const QCommandLineOption AttachmentsOption;
 };
 
 #endif // KEEPASSXC_SHOW_H
