@@ -47,6 +47,15 @@ namespace Tools
                           QProcessEnvironment environment = QProcessEnvironment::systemEnvironment());
     QString cleanFilename(QString filename);
 
+    template <class T> QSet<T> asSet(const QList<T>& a)
+    {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
+        return QSet<T>(a.begin(), a.end());
+#else
+        return QSet<T>::fromList(a);
+#endif
+    }
+
     /**
      * Escapes all characters in regex such that they do not receive any special treatment when used
      * in a regular expression. Essentially, this function escapes any characters not in a-zA-Z0-9.
@@ -118,6 +127,15 @@ namespace Tools
     QVariantMap qo2qvm(const QObject* object, const QStringList& ignoredProperties = {"objectName"});
 
     QString substituteBackupFilePath(QString pattern, const QString& databasePath);
+
+    enum class MimeType : uint8_t
+    {
+        Image,
+        PlainText,
+        Unknown
+    };
+
+    MimeType toMimeType(const QString& mimeName);
 } // namespace Tools
 
 #endif // KEEPASSX_TOOLS_H
