@@ -46,7 +46,7 @@ public:
 
     virtual QString toString() const = 0;
 
-    int benchmark(int msec) const;
+    virtual int benchmark(int msec) const = 0;
 
     /*
      * Default target encryption time, in MS.
@@ -62,13 +62,10 @@ public:
     static const int MAX_ENCRYPTION_TIME = 5000;
 
 protected:
-    virtual int benchmarkImpl(int msec) const = 0;
-
     int m_rounds;
     QByteArray m_seed;
 
 private:
-    class BenchmarkThread;
     const QUuid m_uuid;
 };
 #endif // KEEPASSX_KDF_H
