@@ -19,6 +19,7 @@
 #include "format/Kdbx3Reader.h"
 #include "format/Kdbx4Reader.h"
 #include "format/KeePass1.h"
+#include "keys/CompositeKey.h"
 
 #include <QFile>
 
@@ -71,7 +72,7 @@ bool KeePass2Reader::readDatabase(QIODevice* device, QSharedPointer<const Compos
 
     if (signature1 == KeePass1::SIGNATURE_1 && signature2 == KeePass1::SIGNATURE_2) {
         raiseError(tr("The selected file is an old KeePass 1 database (.kdb).\n\n"
-                      "You can import it by clicking on Database > 'Import KeePass 1 database...'.\n"
+                      "You can import it by clicking on Database > 'Import KeePass 1 database…'.\n"
                       "This is a one-way migration. You won't be able to open the imported "
                       "database with the old KeePassX 0.4 version."));
         return false;
@@ -80,11 +81,8 @@ bool KeePass2Reader::readDatabase(QIODevice* device, QSharedPointer<const Compos
         return false;
     }
 
-    // mask out minor version
-    m_version &= KeePass2::FILE_VERSION_CRITICAL_MASK;
-
-    quint32 maxVersion = KeePass2::FILE_VERSION_4 & KeePass2::FILE_VERSION_CRITICAL_MASK;
-    if (m_version < KeePass2::FILE_VERSION_MIN || m_version > maxVersion) {
+    if (m_version < KeePass2::FILE_VERSION_MIN
+        || (m_version & KeePass2::FILE_VERSION_CRITICAL_MASK) > KeePass2::FILE_VERSION_MAX) {
         raiseError(tr("Unsupported KeePass 2 database version."));
         return false;
     }
@@ -96,7 +94,6 @@ bool KeePass2Reader::readDatabase(QIODevice* device, QSharedPointer<const Compos
         m_reader.reset(new Kdbx4Reader());
     }
 
-    m_reader->setSaveXml(m_saveXml);
     return m_reader->readDatabase(device, std::move(key), db);
 }
 
@@ -110,16 +107,6 @@ QString KeePass2Reader::errorString() const
     return !m_reader.isNull() ? m_reader->errorString() : m_errorStr;
 }
 
-bool KeePass2Reader::saveXml() const
-{
-    return m_saveXml;
-}
-
-void KeePass2Reader::setSaveXml(bool save)
-{
-    m_saveXml = save;
-}
-
 /**
  * @return detected KDBX version
  */
