@@ -25,6 +25,7 @@
 #include "core/Tools.h"
 #include "format/CsvExporter.h"
 #include "gui/Clipboard.h"
+#include "gui/DatabaseIcons.h"
 #include "gui/DatabaseOpenDialog.h"
 #include "gui/DatabaseWidget.h"
 #include "gui/DatabaseWidgetStateSync.h"
@@ -63,7 +64,7 @@ DatabaseTabWidget::DatabaseTabWidget(QWidget* parent)
     // clang-format on
 
 #ifdef Q_OS_MACOS
-    connect(macUtils(), SIGNAL(lockDatabases()), SLOT(lockDatabases()));
+    connect(macUtils(), SIGNAL(userSwitched()), SLOT(lockDatabasesOnUserSwitch()));
 #endif
 
     m_lockDelayTimer.setSingleShot(true);
@@ -276,6 +277,9 @@ DatabaseWidget* DatabaseTabWidget::importFile()
             Merger merger(db.data(), newDb.data());
             merger.setSkipDatabaseCustomData(true);
             merger.merge();
+            // Transfer the root group data
+            newDb->rootGroup()->setName(db->rootGroup()->name());
+            newDb->rootGroup()->setNotes(db->rootGroup()->notes());
             // Show the new database
             auto dbWidget = new DatabaseWidget(newDb, this);
             addDatabaseTab(dbWidget);
@@ -535,21 +539,29 @@ bool DatabaseTabWidget::warnOnExport()
     return ans == MessageBox::Yes;
 }
 
+void DatabaseTabWidget::showDatabaseReports(bool state)
+{
+    if (state) {
+        currentDatabaseWidget()->switchToDatabaseReports();
+    } else {
+        currentDatabaseWidget()->switchToMainView();
+    }
+}
+
+void DatabaseTabWidget::showDatabaseSettings(bool state)
+{
+    if (state) {
+        currentDatabaseWidget()->switchToDatabaseSettings();
+    } else {
+        currentDatabaseWidget()->switchToMainView();
+    }
+}
+
 void DatabaseTabWidget::showDatabaseSecurity()
 {
     currentDatabaseWidget()->switchToDatabaseSecurity();
 }
 
-void DatabaseTabWidget::showDatabaseReports()
-{
-    currentDatabaseWidget()->switchToDatabaseReports();
-}
-
-void DatabaseTabWidget::showDatabaseSettings()
-{
-    currentDatabaseWidget()->switchToDatabaseSettings();
-}
-
 #ifdef WITH_XC_BROWSER_PASSKEYS
 void DatabaseTabWidget::showPasskeys()
 {
@@ -645,6 +657,12 @@ void DatabaseTabWidget::updateTabName(int index)
     index = indexOf(dbWidget);
     setTabText(index, tabName(index));
     setTabToolTip(index, dbWidget->displayFilePath());
+    auto iconIndex = dbWidget->database()->publicIcon();
+    if (iconIndex >= 0 && iconIndex < databaseIcons()->count()) {
+        setTabIcon(index, databaseIcons()->icon(iconIndex));
+    } else {
+        setTabIcon(index, {});
+    }
     emit tabNameChanged();
 }
 
@@ -693,6 +711,13 @@ void DatabaseTabWidget::lockDatabasesDelayed()
     }
 }
 
+void DatabaseTabWidget::lockDatabasesOnUserSwitch()
+{
+    if (config()->get(Config::Security_LockDatabaseOnUserSwitch).toBool()) {
+        lockDatabases();
+    }
+}
+
 /**
  * Unlock a database with an unlock popup dialog.
  *
@@ -782,6 +807,9 @@ void DatabaseTabWidget::handleDatabaseUnlockDialogFinished(bool accepted, Databa
         m_dbWidgetPendingLock = dbWidget;
     }
 
+    // If browser extension requested the unlock make sure cancel is handled
+    m_databaseOpenInProgress = false;
+
     // signal other objects that the dialog finished
     emit databaseUnlockDialogFinished(accepted, dbWidget);
 }
