@@ -18,7 +18,7 @@
 #ifndef KEEPASSX_FONT_H
 #define KEEPASSX_FONT_H
 
-#include <QFont>
+class QFont;
 
 class Font
 {
