@@ -18,6 +18,7 @@
 
 #include "Icons.h"
 
+#include <QBuffer>
 #include <QIconEngine>
 #include <QImageReader>
 #include <QPaintDevice>
@@ -25,6 +26,7 @@
 
 #include "config-keepassx.h"
 #include "core/Config.h"
+#include "core/Database.h"
 #include "gui/DatabaseIcons.h"
 #include "gui/MainWindow.h"
 #include "gui/osutils/OSUtils.h"
