@@ -19,13 +19,10 @@
 #ifndef KEEPASSX_TOTPDIALOG_H
 #define KEEPASSX_TOTPDIALOG_H
 
-#include "core/Database.h"
-#include "core/Entry.h"
-#include "gui/DatabaseWidget.h"
 #include <QDialog>
-#include <QScopedPointer>
-#include <QTimer>
-#include <totp/totp.h>
+
+#include "core/Database.h"
+#include "gui/DatabaseWidget.h"
 
 namespace Ui
 {
