@@ -219,11 +219,12 @@ void EntryView::displaySearch(const QList<Entry*>& entries)
     m_model->setEntries(entries);
     header()->showSection(EntryModel::ParentGroup);
 
+    setFirstEntryActive();
+
     // Reset sort column to 'Group', overrides DatabaseWidgetStateSync
     m_sortModel->sort(EntryModel::ParentGroup, Qt::AscendingOrder);
     sortByColumn(EntryModel::ParentGroup, Qt::AscendingOrder);
 
-    setFirstEntryActive();
     m_inSearchMode = true;
 }
 
@@ -335,6 +336,7 @@ bool EntryView::setViewState(const QByteArray& state)
     bool status = header()->restoreState(state);
     resetFixedColumns();
     m_columnsNeedRelayout = state.isEmpty();
+    onHeaderChanged();
     return status;
 }
 
@@ -375,6 +377,9 @@ void EntryView::toggleColumnVisibility(QAction* action)
     // least one visible column remains, as the table header will disappear
     // entirely when all columns are hidden
     int columnIndex = action->data().toInt();
+    if (columnIndex == EntryModel::Color) {
+        m_model->setBackgroundColorVisible(!action->isChecked());
+    }
     if (action->isChecked()) {
         header()->showSection(columnIndex);
         if (header()->sectionSize(columnIndex) == 0) {
@@ -446,6 +451,8 @@ void EntryView::resetFixedColumns()
             header()->resizeSection(col, width);
         }
     }
+    header()->setMinimumSectionSize(1);
+    header()->resizeSection(EntryModel::Color, ICON_ONLY_SECTION_SIZE);
 }
 
 /**
@@ -474,6 +481,9 @@ void EntryView::resetViewToDefaults()
     header()->hideSection(EntryModel::Attachments);
     header()->hideSection(EntryModel::Size);
     header()->hideSection(EntryModel::PasswordStrength);
+    header()->hideSection(EntryModel::Color);
+    header()->hideSection(EntryModel::ParentGroupPath);
+    onHeaderChanged();
 
     // Reset column order to logical indices
     for (int i = 0; i < header()->count(); ++i) {
@@ -501,6 +511,11 @@ void EntryView::resetViewToDefaults()
     }
 }
 
+void EntryView::onHeaderChanged()
+{
+    m_model->setBackgroundColorVisible(isColumnHidden(EntryModel::Color));
+}
+
 void EntryView::showEvent(QShowEvent* event)
 {
     QTreeView::showEvent(event);
@@ -545,6 +560,8 @@ void EntryView::startDrag(Qt::DropActions supportedActions)
         listWidget.addItem(item);
     }
 
+    listWidget.setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    listWidget.setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     listWidget.setStyleSheet("QListWidget { background-color: palette(highlight); border: 1px solid palette(dark); "
                              "padding: 4px; color: palette(highlighted-text); }");
     auto width = listWidget.sizeHintForColumn(0) + 2 * listWidget.frameWidth();
