@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@
 #include "gui/styles/StateColorPalette.h"
 
 #include <QJsonDocument>
-#include <QJsonObject>
 #include <QMenu>
 #include <QShortcut>
 #include <QSortFilterProxyModel>
@@ -276,10 +275,25 @@ void ReportsWidgetBrowserStatistics::customMenuRequested(QPoint pos)
         });
     }
 
+    // Create the "expire entry" menu item
+    const auto expEntry = new QAction(icons()->icon("entry-expire"), tr("Expire Entry(s)…", "", selected.size()), this);
+    menu->addAction(expEntry);
+    connect(expEntry, &QAction::triggered, this, &ReportsWidgetBrowserStatistics::expireSelectedEntries);
+
     // Create the "delete entry" menu item
-    const auto delEntry = new QAction(icons()->icon("entry-delete"), tr("Delete Entry(s)…", "", selected.size()), this);
-    menu->addAction(delEntry);
-    connect(delEntry, &QAction::triggered, this, &ReportsWidgetBrowserStatistics::deleteSelectedEntries);
+    const auto deleteEntry =
+        new QAction(icons()->icon("entry-delete"), tr("Delete Entry(s)…", "", selected.size()), this);
+    menu->addAction(deleteEntry);
+    connect(deleteEntry, &QAction::triggered, this, &ReportsWidgetBrowserStatistics::deleteSelectedEntries);
+
+    // Create the "delete plugin data" menu item
+    const auto deletePluginData =
+        new QAction(icons()->icon("entry-delete"), tr("Delete plugin data from Entry(s)…", "", selected.size()), this);
+    menu->addAction(deletePluginData);
+    connect(deletePluginData,
+            &QAction::triggered,
+            this,
+            &ReportsWidgetBrowserStatistics::deletePluginDataFromSelectedEntries);
 
     // Create the "exclude from reports" menu item
     const auto exclude = new QAction(icons()->icon("reports-exclude"), tr("Exclude from reports"), this);
@@ -318,7 +332,7 @@ void ReportsWidgetBrowserStatistics::saveSettings()
     // Nothing to do - the tab is passive
 }
 
-void ReportsWidgetBrowserStatistics::deleteSelectedEntries()
+QList<Entry*> ReportsWidgetBrowserStatistics::getSelectedEntries()
 {
     QList<Entry*> selectedEntries;
     for (auto index : m_ui->browserStatisticsTableView->selectionModel()->selectedRows()) {
@@ -328,8 +342,23 @@ void ReportsWidgetBrowserStatistics::deleteSelectedEntries()
             selectedEntries << entry;
         }
     }
+    return selectedEntries;
+}
 
+void ReportsWidgetBrowserStatistics::expireSelectedEntries()
+{
+    for (auto entry : getSelectedEntries()) {
+        entry->expireNow();
+    }
+
+    calculateBrowserStatistics();
+}
+
+void ReportsWidgetBrowserStatistics::deleteSelectedEntries()
+{
+    const auto& selectedEntries = getSelectedEntries();
     bool permanent = !m_db->metadata()->recycleBinEnabled();
+
     if (GuiTools::confirmDeleteEntries(this, selectedEntries, permanent)) {
         GuiTools::deleteEntriesResolveReferences(this, selectedEntries, permanent);
     }
@@ -337,6 +366,18 @@ void ReportsWidgetBrowserStatistics::deleteSelectedEntries()
     calculateBrowserStatistics();
 }
 
+void ReportsWidgetBrowserStatistics::deletePluginDataFromSelectedEntries()
+{
+    const auto& selectedEntries = getSelectedEntries();
+    if (GuiTools::confirmDeletePluginData(this, selectedEntries)) {
+        for (auto& entry : selectedEntries) {
+            browserService()->removePluginData(entry);
+        }
+    }
+
+    calculateBrowserStatistics();
+}
+
 QMap<QString, QStringList> ReportsWidgetBrowserStatistics::getBrowserConfigFromEntry(Entry* entry) const
 {
     QMap<QString, QStringList> configList;
@@ -372,3 +413,17 @@ QMap<QString, QStringList> ReportsWidgetBrowserStatistics::getBrowserConfigFromE
 
     return configList;
 }
+
+QList<Entry*> ReportsWidgetBrowserStatistics::getSelectedEntries() const
+{
+    QList<Entry*> selectedEntries;
+    for (auto index : m_ui->browserStatisticsTableView->selectionModel()->selectedRows()) {
+        auto row = m_modelProxy->mapToSource(index).row();
+        auto entry = m_rowToEntry[row].second;
+        if (entry) {
+            selectedEntries << entry;
+        }
+    }
+
+    return selectedEntries;
+}
