@@ -343,7 +343,7 @@ struct TagsEdit::Impl
         assert(i < tags.size());
         auto occurrencesOfCurrentText =
             std::count_if(tags.cbegin(), tags.cend(), [this](const auto& tag) { return tag.text == currentText(); });
-        if (currentText().isEmpty() || occurrencesOfCurrentText > 1) {
+        if (tags.size() > 1 && (currentText().isEmpty() || occurrencesOfCurrentText > 1)) {
             tags.erase(std::next(tags.begin(), std::ptrdiff_t(editing_index)));
             if (editing_index <= i) { // Do we shift positions after `i`?
                 --i;
@@ -414,6 +414,7 @@ struct TagsEdit::Impl
     void setupCompleter()
     {
         completer->setWidget(ifce);
+        completer->setCaseSensitivity(Qt::CaseInsensitive);
         connect(completer.get(),
                 static_cast<void (QCompleter::*)(QString const&)>(&QCompleter::activated),
                 [this](QString const& text) { currentText(text); });
