@@ -47,9 +47,6 @@ public:
     void initializePage() override;
     bool validatePage() override;
 
-public slots:
-    void toggleAdvancedSettings();
-
 protected:
     QPointer<DatabaseSettingsWidget> m_pageWidget;
     QSharedPointer<Database> m_db;
