@@ -18,12 +18,19 @@
 #ifndef KEEPASSXC_KEESHARE_H
 #define KEEPASSXC_KEESHARE_H
 
-#include <QMap>
+#include <QFileInfo>
 #include <QUuid>
 
 #include "core/Config.h"
 #include "gui/MessageWidget.h"
-#include "keeshare/KeeShareSettings.h"
+
+namespace KeeShareSettings
+{
+    struct Own;
+    struct Active;
+    struct Foreign;
+    struct Reference;
+} // namespace KeeShareSettings
 
 class Group;
 class Database;
@@ -48,12 +55,11 @@ public:
     static QString sharingLabel(const Group* group);
 
     static KeeShareSettings::Own own();
-    static KeeShareSettings::Active active();
-    static KeeShareSettings::Foreign foreign();
-    static void setForeign(const KeeShareSettings::Foreign& foreign);
-    static void setActive(const KeeShareSettings::Active& active);
     static void setOwn(const KeeShareSettings::Own& own);
 
+    static KeeShareSettings::Active active();
+    static void setActive(const KeeShareSettings::Active& active);
+
     static KeeShareSettings::Reference referenceOf(const Group* group);
     static void setReferenceTo(Group* group, const KeeShareSettings::Reference& reference);
     static QString referenceTypeLabel(const KeeShareSettings::Reference& reference);
