@@ -20,8 +20,6 @@
 
 #include <QMap>
 #include <QObject>
-#include <QSharedPointer>
-#include <QStringList>
 
 #include "gui/MessageWidget.h"
 #include "keeshare/KeeShareSettings.h"
@@ -83,6 +81,7 @@ private:
     QMap<QPointer<Group>, KeeShareSettings::Reference> m_groupToReference;
     QMap<QString, QPointer<Group>> m_shareToGroup;
     QMap<QString, QSharedPointer<FileWatcher>> m_fileWatchers;
+    bool m_inFileUpdate = false;
 };
 
 #endif // KEEPASSXC_SHAREOBSERVER_H
