@@ -16,16 +16,16 @@
  *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef ASN1KEY_H
-#define ASN1KEY_H
+#ifndef KEEPASSXC_ASN1KEY_H
+#define KEEPASSXC_ASN1KEY_H
 
-#include "OpenSSHKey.h"
-#include <QtCore>
+class QByteArray;
+class OpenSSHKey;
 
 namespace ASN1Key
 {
     bool parseDSA(QByteArray& ba, OpenSSHKey& key);
     bool parseRSA(QByteArray& ba, OpenSSHKey& key);
-}
+} // namespace ASN1Key
 
-#endif // ASN1KEY_H
+#endif // KEEPASSXC_ASN1KEY_H
