@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2021 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2025 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -36,25 +36,23 @@ private slots:
     void testDecryptMessage();
     void testGetBase64FromKey();
     void testIncrementNonce();
-
-    void testTopLevelDomain();
-    void testIsIpAddress();
+    void testBuildResponse();
     void testSortPriority();
     void testSortPriority_data();
     void testSearchEntries();
     void testSearchEntriesByPath();
     void testSearchEntriesByUUID();
+    void testSearchEntriesByReference();
     void testSearchEntriesWithPort();
     void testSearchEntriesWithAdditionalURLs();
+    void testSearchEntriesWithWildcardURLs();
     void testInvalidEntries();
     void testSubdomainsAndPaths();
-    void testValidURLs();
     void testBestMatchingCredentials();
     void testBestMatchingWithAdditionalURLs();
-    void testIsUrlIdentical();
 
 private:
-    QList<Entry*> createEntries(QStringList& urls, Group* root) const;
+    QList<Entry*> createEntries(QStringList& urls, Group* root, bool additionalUrl = false) const;
     void compareEntriesByPath(QSharedPointer<Database> db, QList<Entry*> entries, QString path);
 
     QScopedPointer<BrowserAction> m_browserAction;
