@@ -36,6 +36,8 @@ private slots:
     void testResolveRecursivePlaceholders();
     void testResolveReferencePlaceholders();
     void testResolveNonIdPlaceholdersToUuid();
+    void testResolveConversionPlaceholders();
+    void testResolveReplacePlaceholders();
     void testResolveClonedEntry();
     void testIsRecycled();
     void testMoveUpDown();
