@@ -24,7 +24,7 @@ class Database;
 
 class TestKdbx2 : public QObject
 {
-Q_OBJECT
+    Q_OBJECT
 
 private slots:
     void initTestCase();
@@ -32,7 +32,7 @@ private slots:
     void testFormat200Upgrade();
 
 private:
-    void verifyKdbx2Db(Database* db);
+    void verifyKdbx2Db(QSharedPointer<Database> db);
 };
 
 #endif // KEEPASSXC_TEST_KDBX2_H
