@@ -26,12 +26,15 @@ class TestModified : public QObject
 
 private slots:
     void initTestCase();
+    void init();
+    void cleanup();
     void testSignals();
     void testGroupSets();
     void testEntrySets();
     void testHistoryItems();
     void testHistoryMaxSize();
     void testCustomData();
+    void testBlockModifiedSignal();
 };
 
 #endif // KEEPASSX_TESTMODIFIED_H
