@@ -120,8 +120,8 @@ class FakeClient : public DBusClient
 public:
     explicit FakeClient(DBusMgr* dbus)
         : DBusClient(
-            dbus,
-            {QStringLiteral("local"), 0, true, {ProcInfo{0, 0, QStringLiteral("fake-client"), QString{}, QString{}}}})
+              dbus,
+              {QStringLiteral("local"), 0, true, {ProcInfo{0, 0, QStringLiteral("fake-client"), QString{}, QString{}}}})
     {
     }
 };
@@ -1879,6 +1879,8 @@ bool TestGuiFdoSecrets::driveNewDatabaseWizard()
             tmpFile.close();
             fileDialog()->setNextFileName(tmpFile.fileName());
 
+            // click Continue on the warning due to weak password
+            MessageBox::setNextAnswer(MessageBox::ContinueWithWeakPass);
             wizard->accept();
 
             tmpFile.remove();
