@@ -28,14 +28,17 @@ class MockChallengeResponseKey : public ChallengeResponseKey
 {
 public:
     explicit MockChallengeResponseKey(const QByteArray& secret);
-    Q_DISABLE_COPY(MockChallengeResponseKey);
-    ~MockChallengeResponseKey() override;
+    ~MockChallengeResponseKey() override = default;
+
     QByteArray rawKey() const override;
+
     bool challenge(const QByteArray& challenge) override;
 
 private:
     QByteArray m_challenge;
     QByteArray m_secret;
+
+    Q_DISABLE_COPY(MockChallengeResponseKey);
 };
 
 #endif // KEEPASSXC_MOCKCHALLENGERESPONSEKEY_H
